#ifndef ASSETS_H
#define ASSETS_H

struct ASSET
{
	const char** mono;
	const char** shade;
	const char** color;
};

extern ASSET avatar;
extern ASSET ball1;
extern ASSET ball2;
extern ASSET ball3;
extern ASSET ball4;
extern ASSET base;
extern ASSET basher;
extern ASSET book;
extern ASSET boom;
extern ASSET cannon;
extern ASSET character;
extern ASSET chassis;
extern ASSET city;
extern ASSET co_boom;
extern ASSET creature;
extern ASSET dialog_left;
extern ASSET dialog_right;
extern ASSET digits_large;
extern ASSET drone;
extern ASSET dunes;
extern ASSET flame1;
extern ASSET flame1_mask;
extern ASSET flame2;
extern ASSET flame2_mask;
extern ASSET flame3;
extern ASSET flame3_mask;
extern ASSET fly_boom1;
extern ASSET fly_boom2;
extern ASSET game_over_bk;
extern ASSET game_over_fg;
extern ASSET game_over_tx;
extern ASSET ground_boom;
extern ASSET heap1;
extern ASSET heap2;
extern ASSET heap3;
extern ASSET keyb;
extern ASSET land_mine;
extern ASSET map;
extern ASSET menu_bkg;
extern ASSET menu_car;
extern ASSET menu_frm;
extern ASSET menu_map;
extern ASSET menu_mod;
extern ASSET menu_sld;
extern ASSET menu_swt;
extern ASSET menu_tag;
extern ASSET menu_wnd;
extern ASSET mountains;
extern ASSET perf_horz;
extern ASSET perf_icon;
extern ASSET perf_line;
extern ASSET perf_prgs;
extern ASSET perf_tick;
extern ASSET rocket;
extern ASSET scene_ascii;
extern ASSET scene_bk_wheel;
extern ASSET scene_chassis;
extern ASSET scene_fr_wheel;
extern ASSET scene_patrol;
extern ASSET tank;
extern ASSET tank_bullet;
extern ASSET ufo;
extern ASSET wheel;

#endif

